//=============================================================================
// Mutator for the "TickyTicker" PhysX clock map.
// Provides, via the Mutate function, the ability to set the time on the clock
// and adjust its rate by changing the height of the pendulum.
// http://mrevil.pwp.blueyonder.co.uk/unreal/
//=============================================================================
class UTMutator_HandRotator extends UTMutator;

var Actor HourHand, MinuteHand, SecondHand;
var RB_ConstraintActor PendulumHinge;

function PostBeginPlay()
{
	local Actor A;

	super.PostBeginPlay();

	foreach AllActors(class'Actor', A)
	{
		switch(A.Tag)
		{
			case 'HourHand':
				HourHand = A;
				break;
			case 'MinuteHand':
				MinuteHand = A;
				break;
			case 'SecondHand':
				SecondHand = A;
				break;
			case 'PendulumHinge':
				PendulumHinge = RB_ConstraintActor(A);
				break;
		}
	}

	`log("Hands found:"@HourHand@MinuteHand@SecondHand);
	`log("Pendulum hinge found:"@PendulumHinge);
}

/**
 * To change the time, type "mutate setclock hh:mm" where hh is an hour from 0-12 and mm is a minute from 0-59. Seconds will be zeroed when time is set.
 * To change the rate, type "mutate adjustpendulum height" where height is a positive number to lower the pendulum (slow down clock) or a negative number to raise it (speed up clock).
 */
function Mutate(string MutateString, PlayerController Sender)
{
	local array<string> TimeStrings;

	super.Mutate(MutateString, Sender);

	MutateString = Locs(MutateString);

	if(Left(MutateString, 8) == "setclock")
	{
		MutateString -= "setclock";
		ParseStringIntoArray(MutateString, TimeStrings, ":", false);

		if(TimeStrings.length == 2)
		{
			SetClock(int(TimeStrings[0]), int(TimeStrings[1]));
		}
		else
		{
			`log("Incorrectly formatted time entered:"@MutateString);
		}
	}
	else if(Left(MutateString, 14) == "adjustpendulum")
	{
		MutateString -= "adjustpendulum";
		AdjustPendulum(float(MutateString));
	}
}

function SetClock(int Hours, int Minutes)
{
	local float HourHandAngle, MinuteHandAngle, SecondHandAngle;

	if(Hours > 12 || Hours < 0 || Minutes > 59 || Minutes < 0)
	{
		`log("Invalid time entered:"@Hours@":"@Minutes);
		return;
	}

	`log("Setting time to"@Hours@":"@Minutes);

	HourHandAngle = (65536.0 / 12.0) * Hours;
	HourHandAngle += (65536.0 / 12.0) * (Minutes / 60.0);

	MinuteHandAngle = (65536.0 / 60.0) * Minutes;

	SecondHandAngle = 0.0;

	RotateHand(HourHand, HourHandAngle);
	RotateHand(MinuteHand, MinuteHandAngle);
	RotateHand(SecondHand, SecondHandAngle);
}

function RotateHand(Actor Hand, int HandAngle)
{
	local Rotator R;
	local Actor HandBase;

	if(Hand != none)
	{
		HandBase = Hand.Base;
		Hand.SetBase(none);

		R.Roll = -HandAngle; //n.b. angle is reversed for some reason.
		Hand.SetRotation(R);

		Hand.SetBase(HandBase);
	}
}

function AdjustPendulum(float NewHeight)
{
	local Vector NewPosition;

	if(PendulumHinge != none)
	{
		`log("Adjusting pendulum height to:"@NewHeight);
		NewPosition.Z = NewHeight;
		PendulumHinge.ConstraintInstance.SetLinearPositionTarget(NewPosition);
	}
}

defaultproperties
{
}